@@ -114,3 +114,55 @@ def approximate_hyperbolic_trig(expr, method='pade', order=1, numer_order=None):
     expr = expand_hyperbolic_trig(expr)
     return approximate_exp(expr, method=method, order=order,
                            numer_order=numer_order)
+
+
+def approximate_dominant_terms(expr, defs, threshold=0.01):
+
+    terms = expr.as_ordered_terms()
+
+    # Cannot substitute first since SymPy reorders the args.
+
+    absvalmax = None
+    for term in terms:
+        nterm = term.subs(defs)
+        if not nterm.is_constant():
+            continue
+        absval = abs(nterm)
+        if absvalmax is None or absval > absvalmax:
+            absvalmax = absval
+
+    newexpr = 0
+    for term in terms:
+        nterm = term.subs(defs)
+        absval = abs(nterm)
+        if (not nterm.is_constant()) or (absval > threshold * absvalmax):
+            newexpr += term
+
+    return newexpr
+
+
+def approximate_dominant(expr, defs, threshold=0.01):
+    """Approximate expression using ball-park numerical values for the
+    symbols to decide which terms in a sum dominate the sum.  A term
+    is neglected if its absolute value is below `threshold` times the
+    maximum absolute value of the terms in the sum.
+
+    `defs` is a dict, set, list, or tuple of symbol definitions."""
+
+    def query(expr):
+
+        return expr.is_Add
+
+    def value(expr):
+        return approximate_dominant_terms(expr, defs, threshold)
+
+    return expr.replace(query, value)
+
+
+def approximate_order(expr, var, order):
+    """Approximate expression by reducing order of polynomial to
+    specified order."""
+
+    from sympy import O
+
+    return (expr.expand() + O(var ** (order + 1))).removeO()
