@@ -6,7 +6,7 @@ Copyright 2022 Michael Hayes, UCECE
 
 from .expr import expr, equation, ExprTuple
 from .differentialequation import DifferentialEquation
-from .functions import Derivative
+from .functions import Derivative, Function
 from .texpr import TimeDomainExpression
 from .symbols import t, s
 from .utils import isiterable
@@ -83,7 +83,7 @@ class LTIFilter(object):
     def frequency_response(self, **assumptions):
         """Return frequency response."""
 
-        return self.transfer_function().FT(**assumptions)
+        return self.transfer_function().frequency_response(**assumptions)
 
     def differential_equation(self, inputsym='x', outputsym='y'):
         """Return differential equation."""
@@ -91,26 +91,34 @@ class LTIFilter(object):
         rhs = 0 * t
 
         for m, bn in enumerate(self.b):
-            rhs += bn * Derivative(expr('%s(t)' % inputsym), t, m)
+            rhs += bn * Derivative(Function(inputsym)(t), t, m)
 
         for m, an in enumerate(self.a[1:]):
-            rhs -= an * Derivative(expr('%s(t)' % outputsym), t, m + 1)
+            rhs -= an * Derivative(Function(outputsym)(t), t, m + 1)
 
         lhs = self.a[0] * expr('y(t)')
         return DifferentialEquation(lhs, rhs, inputsym, outputsym)
 
-    def sdomain_initial_response(self, ic):
+    def sdomain_initial_response(self, ic=None):
         """Return s-domain response due to initial conditions.
-           ic : list of initial conditions y(0), y'(0), y''(0), ...
+           ic : list or tuple of initial conditions y(0), y'(0), y''(0), ...
 
         """
 
         from .sym import ssym
 
+        if ic is None:
+            ic = ()
+
+        if not isiterable(ic):
+            ic = (ic, )
+
+        ic = ExprTuple(ic)
+
         Nl = len(self.a)
-        if len(ic) != Nl:
+        if len(ic) != Nl - 1:
             raise ValueError('Expecting %d initial conditions, got %d'
-                             % (Nl, len(ic)))
+                             % (Nl - 1, len(ic)))
 
         # Denominator for Yi(s)
         denom = self.a[0] * s**0
@@ -136,7 +144,7 @@ class LTIFilter(object):
 
         return Ysi
 
-    def initial_response(self, ic):
+    def initial_response(self, ic=None):
         """Return response due to initial conditions in the time domain.
            ic : list of initial conditions y(0), y'(0), y''(0), ...
         """
