@@ -2,7 +2,7 @@
 This module defines the components for modified nodal analysis.  The components
 are defined at the bottom of this file.
 
-Copyright 2015--2022 Michael Hayes, UCECE
+Copyright 2015--2023 Michael Hayes, UCECE
 
 """
 
@@ -19,6 +19,7 @@ from .superpositioncurrent import SuperpositionCurrent
 from .voltage import voltage
 from .current import current
 from .opts import Opts
+from .node import DummyNode
 import lcapy
 import inspect
 import sys
@@ -55,32 +56,42 @@ class Cpt(ImmittanceMixin):
     is_transformer = False
     is_wire = False
 
-    def __init__(self, cct, namespace, defname, name, cpt_type, cpt_id, string,
-                 opts_string, nodes, keyword, *args):
+    def __init__(self, cct, namespace, name, cpt_type, cpt_id, string,
+                 opts_string, node_names, keyword, *args):
 
         self.cct = cct
         self.type = cpt_type
         self.id = cpt_id
-        self.defname = defname
         self.name = name
         self.relname = name
         self.namespace = ''
-        self.nodenames = nodes
-        self.relnodes = nodes
+        self.node_names = node_names
 
+        self.nodes = []
+        if cct is not None:
+            for node_name in node_names:
+                self.nodes.append(cct.nodes.add(node_name, self, cct))
+        else:
+            for node_name in node_names:
+                self.nodes.append(DummyNode(node_name))
+
+        self.relnodes = self.nodes
+
+        # Handle names such as a.R1 or a.b.R1.  The namespace is
+        # a for the first example and a.b for the second.
+        # The relative name (relname) in both cases is R1.
         parts = name.split('.')
         if len(parts) > 1:
             self.namespace = '.'.join(parts[0:-1]) + '.'
             self.relname = parts[-1]
-            self.relnodes = []
-            for node in nodes:
-                if node.startswith(self.namespace):
-                    node = node[len(self.namespace):]
-                self.relnodes.append(node)
+            # self.relnodes = []
+            # for node in nodes:
+            #     if node.startswith(self.namespace):
+            #         node = node[len(self.namespace):]
+            #     self.relnodes.append(node)
 
         self._string = string
-        # self.net = string.split(';')[0]
-        self.args = args
+        self.args = list(args)
         self.classname = self.__class__.__name__
         self.keyword = keyword
         self.opts = Opts(opts_string)
@@ -116,13 +127,25 @@ class Cpt(ImmittanceMixin):
         return self.__str__()
 
     def __str__(self):
-        # TODO, use self._netmake() but fix for XX
-        return self._string
+        if self.type == 'XX':
+            return self._string
+        else:
+            # This works if the nodes are renamed or the args are changed
+            return self._netmake()
 
     @property
     def cpt(self):
         return self._cpt
 
+    @property
+    def _kind(self):
+        return self.keyword[1]
+
+    def pdb(self):
+        import pdb
+        pdb.set_trace()
+        return self
+
     def check(self):
         pass
 
@@ -196,18 +219,30 @@ class Cpt(ImmittanceMixin):
 
         return self._copy()
 
+    def _change_kind(self, kind):
+
+        if self.keyword[1] == kind:
+            return self
+
+        cpt = self
+        self.cct.remove(cpt.name)
+        cpt.keyword = (cpt.keyword[0], kind)
+        self.cct.add(str(cpt))
+        return self.cct[cpt.name]
+
     def _netsubs(self, node_map=None, zero=False, subs_dict=None):
         """Create a new net description using substitutions in `subs_dict`.
         If `node_map` is not `None`, rename the nodes.  If `zero` is `True`,
         set args to zero."""
 
-        string = self.defname
+        string = self.name
         field = 0
 
         for node in self.relnodes:
+            node_name = node.name
             if node_map is not None:
-                node = node_map[node]
-            string += ' ' + node
+                node_name = node_map[node_name]
+            string += ' ' + node_name
             field += 1
             if field == self.keyword[0]:
                 string += ' ' + self.keyword[1]
@@ -250,25 +285,40 @@ class Cpt(ImmittanceMixin):
 
         if args is None:
             args = self.args
-
-        if not isinstance(args, tuple):
-            args = (args, )
+        elif isinstance(args, tuple):
+            args = list(args)
+        elif not isinstance(args, list):
+            args = [args]
 
         if opts is None:
             opts = self.opts
 
         fmtargs = []
         for arg in args:
+            # Ignore initial value if not defined.
+            if arg is None:
+                continue
             fmtargs.append(self._arg_format(arg))
 
         if len(fmtargs) == 1 and fmtargs[0] == name:
             fmtargs = []
 
+        # Never name wires, etc, since this confuses the autonamer
+        if name[0] in ('A', 'O', 'W', 'P') and name[1:].startswith('anon'):
+            name = name[0]
+
         parts = [name]
+
+        # Handle things like U1 opamp
+        if self.keyword[0] == 0 and self.keyword[1] != '':
+            parts.append(self.keyword[1])
+
         for m, node in enumerate(nodes):
-            parts.append(node)
-            if not ignore_keyword and self.keyword[0] == m + 1:
+            parts.append(node.name)
+            if not ignore_keyword and self.keyword[0] == m + 1 \
+               and self.keyword[1] != '':
                 parts.append(self.keyword[1])
+
         parts.extend(fmtargs)
 
         net = ' '.join(parts)
@@ -365,22 +415,22 @@ class Cpt(ImmittanceMixin):
                 return False
         return True
 
-    @ property
+    @property
     def is_source(self):
         """Return True if component is a source (dependent or independent)"""
         return self.dependent_source or self.independent_source
 
-    @ property
+    @property
     def is_dependent_source(self):
         """Return True if component is a dependent source"""
         return self.dependent_source
 
-    @ property
+    @property
     def is_independent_source(self):
         """Return True if component is an independent source"""
         return self.independent_source
 
-    @ property
+    @property
     def _source_IV(self):
 
         if self.cpt.is_voltage_source:
@@ -403,7 +453,7 @@ class Cpt(ImmittanceMixin):
 
         return self.cct.in_series(self.name)
 
-    @ property
+    @property
     def is_causal(self):
         """Return True if causal component or if source produces
         a causal signal."""
@@ -412,55 +462,55 @@ class Cpt(ImmittanceMixin):
             return self._source_IV.is_causal
         return self.cpt.is_causal
 
-    @ property
+    @property
     def is_dc(self):
         """Return True if source is dc."""
 
         return self._source_IV.is_dc
 
-    @ property
+    @property
     def is_ac(self):
         """Return True if source is ac."""
 
         return self._source_IV.is_ac
 
-    @ property
+    @property
     def has_ac(self):
         """Return True if source has ac component."""
 
         return self._source_IV.has_ac
 
-    @ property
+    @property
     def has_dc(self):
         """Return True if source has dc component."""
 
         return self._source_IV.has_dc
 
-    @ property
+    @property
     def has_noisy(self):
         """Return True if source has noisy component."""
 
         return self._source_IV.has_noisy
 
-    @ property
+    @property
     def has_s_transient(self):
         """Return True if source has transient component defined in s-domain."""
 
         return self._source_IV.has_s_transient
 
-    @ property
+    @property
     def has_t_transient(self):
         """Return True if source has transient component defined in time domain."""
 
         return self._source_IV.has_t_transient
 
-    @ property
+    @property
     def has_transient(self):
         """Return True if source has a transient component."""
 
         return self._source_IV.has_transient
 
-    @ property
+    @property
     def is_noisy(self):
         """Return True if source is noisy."""
 
@@ -471,82 +521,82 @@ class Cpt(ImmittanceMixin):
         else:
             raise ValueError('%s is not a source' % self)
 
-    @ property
+    @property
     def is_noiseless(self):
         """Return True if component is noiseless."""
 
         return self.cpt.is_noiseless
 
-    @ property
+    @property
     def is_inductor(self):
         """Return True if component is an inductor."""
         return self.cpt.is_inductor
 
-    @ property
+    @property
     def is_capacitor(self):
         """Return True if component is a capacitor."""
         return self.cpt.is_capacitor
 
-    @ property
+    @property
     def is_reactance(self):
         """Return True if component is a capacitor or inductor."""
         return self.is_capacitor or self.is_inductor
 
-    @ property
+    @property
     def is_resistor(self):
         """Return True if component is a resistor."""
         return self.cpt.is_resistor
 
-    @ property
+    @property
     def is_conductor(self):
         """Return True if component is a conductor."""
         return self.cpt.is_conductor
 
-    @ property
+    @property
     def is_voltage_source(self):
         """Return True if component is a voltage source (dependent or
         independent)"""
         return self.cpt.is_voltage_source
 
-    @ property
+    @property
     def is_dependent_voltage_source(self):
         """Return True if component is a dependent voltage source."""
         return self.cpt.is_voltage_source and self.dependent_source
 
-    @ property
+    @property
     def is_independent_voltage_source(self):
         """Return True if component is a independent voltage source."""
         return self.cpt.is_voltage_source and self.independent_source
 
-    @ property
+    @property
     def is_current_source(self):
         """Return True if component is a current source (dependent or
         independent)"""
         return self.cpt.is_current_source
 
-    @ property
+    @property
     def is_dependent_current_source(self):
         """Return True if component is a dependent current source."""
         return self.cpt.is_current_source and self.dependent_source
 
-    @ property
+    @property
     def is_independent_current_source(self):
         """Return True if component is a independent current source."""
         return self.cpt.is_current_source and self.independent_source
 
-    @ property
+    @property
     def zeroic(self):
         """Return True if initial conditions are zero (or unspecified)."""
 
         return self.cpt.zeroic
 
-    @ property
+    @property
     def has_ic(self):
         """Return True if initial conditions are specified."""
 
         return self.cpt.has_ic
 
-    @ property
+    @property
     def I(self):
         """Current through component.  The current is defined to be into the
         positive node for passive devices and out of the positive node
@@ -554,7 +604,7 @@ class Cpt(ImmittanceMixin):
 
         return self.cct.get_I(self.name)
 
-    @ property
+    @property
     def i(self):
         """Time-domain current through component.  The current is
         defined to be into the positive node for passive devices and
@@ -562,57 +612,57 @@ class Cpt(ImmittanceMixin):
 
         return self.cct.get_i(self.name)
 
-    @ property
+    @property
     def V(self):
         """Voltage drop across component."""
 
-        return self.cct.get_Vd(self.nodenames[0], self.nodenames[1])
+        return self.cct.get_Vd(self.nodes[0].name, self.nodes[1].name)
 
-    @ property
+    @property
     def v(self):
         """Time-domain voltage drop across component."""
 
-        return self.cct.get_vd(self.nodenames[0], self.nodenames[1])
+        return self.cct.get_vd(self.nodes[0].name, self.nodes[1].name)
 
-    @ property
+    @property
     def Isc(self):
         """Short-circuit current for component in isolation, i.e, current in
         wire connected across component."""
 
         return self.cpt.Isc.select(self.cct.kind)
 
-    @ property
+    @property
     def isc(self):
         """Short-circuit time-domain current for component in isolation, i.e,
         current in wire connected across component."""
 
         return self.Isc.time()
 
-    @ property
+    @property
     def Voc(self):
         """Open-circuit voltage for component in isolation."""
 
         return self.cpt.Voc.select(self.cct.kind)
 
-    @ property
+    @property
     def voc(self):
         """Open-circuit time-domain voltage for component in isolation."""
 
         return self.Voc.time()
 
-    @ property
+    @property
     def V0(self):
         """Initial voltage (for capacitors only)."""
 
         return voltage(0)
 
-    @ property
+    @property
     def I0(self):
         """Initial current (for inductors only)."""
 
         return current(0)
 
-    @ property
+    @property
     def admittance(self):
         """Self admittance of component.
 
@@ -624,7 +674,7 @@ class Cpt(ImmittanceMixin):
 
         return self.cpt.admittance._select(self.cct.kind)
 
-    @ property
+    @property
     def impedance(self):
         """Self impedance of component.
 
@@ -636,7 +686,7 @@ class Cpt(ImmittanceMixin):
 
         return self.cpt.impedance._select(self.cct.kind)
 
-    @ property
+    @property
     def dpIsc(self):
         """Driving-point short-circuit current, i.e, current in wire
         connected across component connected in-circuit.
@@ -644,58 +694,58 @@ class Cpt(ImmittanceMixin):
         """
         return self.oneport().Isc
 
-    @ property
+    @property
     def dpisc(self):
         """Driving-point short-circuit time-domain current i.e, current in
         wire connected across component in-circuit."""
 
         return self.dpIsc.time()
 
-    @ property
+    @property
     def dpVoc(self):
         """Driving-point open-circuit voltage across component in-circuit."""
 
         return self.oneport().V
 
-    @ property
+    @property
     def dpvoc(self):
         """Driving-point open-circuit time-domain voltage across component in
         circuit."""
 
         return self.dpVoc.time()
 
-    @ property
+    @property
     def dpY(self):
         """Driving-point admittance measured across component in-circuit.  For
         the admittance of the component in isolation use .Y"""
 
-        return self.cct.admittance(self.nodenames[1], self.nodenames[0])
+        return self.cct.admittance(self.nodes[1].name, self.nodes[0].name)
 
-    @ property
+    @property
     def dpZ(self):
         """Driving-point impedance measured across component in-circuit.  For
         the impedance of the component in isolation use .Z"""
 
-        return self.cct.impedance(self.nodenames[1], self.nodenames[0])
+        return self.cct.impedance(self.nodes[1].name, self.nodes[0].name)
 
     def _dummy_node(self):
 
-        return self.cct._dummy_node()
+        return DummyNode(self.cct._dummy_node_name())
 
     def oneport(self):
         """Create oneport object."""
 
-        return self.cct.oneport(self.nodenames[1], self.nodenames[0])
+        return self.cct.oneport(self.nodes[1].name, self.nodes[0].name)
 
     def thevenin(self):
         """Create Thevenin oneport object."""
 
-        return self.cct.thevenin(self.nodenames[1], self.nodenames[0])
+        return self.cct.thevenin(self.nodes[1].name, self.nodes[0].name)
 
     def norton(self):
         """Create Norton oneport object."""
 
-        return self.cct.norton(self.nodenames[1], self.nodenames[0])
+        return self.cct.norton(self.nodes[1].name, self.nodes[0].name)
 
     def transfer(self, cpt):
         """Create transfer function for the s-domain voltage across the
@@ -704,15 +754,8 @@ class Cpt(ImmittanceMixin):
         if isinstance(cpt, str):
             cpt = self.cct._elements[cpt]
 
-        return self.cct.transfer(self.nodenames[1], self.nodenames[0],
-                                 cpt.nodenames[1], cpt.nodenames[0])
-
-    @ property
-    def nodes(self):
-        """Return list of nodes for this component.   See also
-        nodenames."""
-
-        return [self.cct.nodes[nodename] for nodename in self.nodenames]
+        return self.cct.transfer(self.nodes[1].name, self.nodes[0].name,
+                                 cpt.nodes[1].name, cpt.nodes[0].name)
 
     def connected(self):
         """Return list of components connected to this component."""
@@ -766,7 +809,7 @@ class Cpt(ImmittanceMixin):
         # Could add wire or zero ohm resistor but then could not
         # determine current through the short.  So instead add a
         # voltage source.
-        self.cct.add('V? %s %s 0' % (self.nodes[0].name, self.nodes[1].name))
+        self.cct.add('V? %s %s 0' % (self.nodes[0], self.nodes[1]))
 
         return self.cct.last_added()
 
@@ -860,7 +903,9 @@ class DependentSource(Dummy):
 
 class RLC(Cpt):
 
-    def _s_model(self, var):
+    def _s_model(self, kind):
+
+        var = s if kind == 's' else kind
 
         if self.Voc == 0:
             return self._netmake_variant('Z', args=self.Z(var))
@@ -993,12 +1038,14 @@ class C(RC):
 
         opts.strip_voltage_labels()
         rnet = self._netmake_variant('R', suffix='eq',
-                                     nodes=(self.relnodes[0], dummy_node),
+                                     nodes=(
+                                         self.relnodes[0], dummy_node),
                                      args=Req, opts=opts)
 
         opts.strip_current_labels()
         vnet = self._netmake_variant('V', suffix='eq',
-                                     nodes=(dummy_node, self.relnodes[1]),
+                                     nodes=(dummy_node,
+                                            self.relnodes[1]),
                                      args=('dc', Veq), opts=opts)
 
         # TODO: the voltage labels should be added across an
@@ -1035,7 +1082,7 @@ class VCVS(DependentSource):
 
     def check(self):
 
-        n1, n2, n3, n4 = self.nodenames
+        n1, n2, n3, n4 = [node.name for node in self.nodes]
         if n1 == n3 and n2 == n4:
             warn('VCVS output nodes and control nodes are the same for %s' % self)
 
@@ -1115,14 +1162,18 @@ class Efdopamp(DependentSource):
         Ad = '{%s / 2}' % Ad
 
         opampp = self._netmake_expand('Ep',
-                                      nodes=(self.relnodes[0], self.relnodes[4],
+                                      nodes=(self.relnodes[0],
+                                             self.relnodes[4],
                                              'opamp',
-                                             self.relnodes[2], self.relnodes[3]),
+                                             self.relnodes[2],
+                                             self.relnodes[3]),
                                       args=(Ad, Ac))
         opampm = self._netmake_expand('Em',
-                                      nodes=(self.relnodes[4], self.relnodes[1],
+                                      nodes=(self.relnodes[4],
+                                             self.relnodes[1],
                                              'opamp',
-                                             self.relnodes[2], self.relnodes[3]),
+                                             self.relnodes[2],
+                                             self.relnodes[3]),
                                       args=(Ad, Ac))
 
         return opampp + '\n' + opampm + '\n'
@@ -1144,7 +1195,8 @@ class Einamp(DependentSource):
 
         cpts.append(self._netmake_expand('Ep',
                                          nodes=(node7, '0', 'opamp',
-                                                self.relnodes[2], self.relnodes[4]),
+                                                self.relnodes[2],
+                                                self.relnodes[4]),
                                          args=(Ad, )))
         cpts.append(self._netmake_expand('Em',
                                          nodes=(node8, '0', 'opamp',
@@ -1229,7 +1281,7 @@ class GY(Dummy):
     def _stamp(self, mna):
 
         n1, n2, n3, n4 = mna._cpt_node_indexes(self)
-        m1 = mna._branch_index(self.defname + 'X')
+        m1 = mna._branch_index(self.name + 'X')
         m2 = mna._cpt_branch_index(self)
 
         # m1 is the input branch
@@ -1272,7 +1324,7 @@ class TVtriode(Dummy):
     def _stamp(self, cct):
 
         n1, n2, n3 = self.node_indexes
-        m1 = self.cct._branch_index(self.defname + 'X')
+        m1 = self.cct._branch_index(self.name + 'X')
         m2 = self.branch_index
 
         # m1 is the input branch
@@ -1362,8 +1414,8 @@ class I(IndependentSource):
     def _ss_model(self):
         return self._netmake(args='%s(t)' % self.relname.lower())
 
-    def _s_model(self, var):
-        return self._netmake(args=self.Isc.laplace()(var))
+    def _s_model(self, kind):
+        return self._netmake(args=self.Isc.laplace(), ignore_keyword=True)
 
     def _pre_initial_model(self):
 
@@ -1372,12 +1424,12 @@ class I(IndependentSource):
 
 class K(Dummy):
 
-    def __init__(self, cct, namespace, defname, name, cpt_type, cpt_id, string,
+    def __init__(self, cct, namespace, name, cpt_type, cpt_id, string,
                  opts_string, nodes, keyword, *args):
 
         self.Lname1 = args[0]
         self.Lname2 = args[1]
-        super(K, self).__init__(cct, namespace, defname, name,
+        super(K, self).__init__(cct, namespace, name,
                                 cpt_type, cpt_id, string,
                                 opts_string, nodes, keyword, *args)
 
@@ -1430,12 +1482,14 @@ class L(RLC):
 
         opts.strip_voltage_labels()
         rnet = self._netmake_variant('R', suffix='eq',
-                                     nodes=(self.relnodes[0], dummy_node),
+                                     nodes=(
+                                         self.relnodes[0], dummy_node),
                                      args=Req, opts=opts)
 
         opts.strip_current_labels()
         vnet = self._netmake_variant('V', suffix='eq',
-                                     nodes=(dummy_node, self.relnodes[1]),
+                                     nodes=(dummy_node,
+                                            self.relnodes[1]),
                                      args=('dc', Veq), opts=opts)
 
         # TODO: the voltage labels should be added across an
@@ -1695,8 +1749,12 @@ class SW(TimeVarying):
             net = self._netmake_opts(opts)
 
             if active ^ ('mirror' in self.opts):
-                return net + '\n' + self._netmake_W(nodes=(self.relnodes[0], self.relnodes[2]), opts=wopts)
-            return net + '\n' + self._netmake_W(nodes=(self.relnodes[0], self.relnodes[1]), opts=wopts)
+                return net + '\n' + self._netmake_W(nodes=(self.relnodes[0],
+                                                           self.relnodes[2]),
+                                                    opts=wopts)
+            return net + '\n' + self._netmake_W(nodes=(self.relnodes[0],
+                                                       self.relnodes[1]),
+                                                opts=wopts)
 
         else:
             raise RuntimeError('Internal error, unhandled switch %s' % self)
@@ -2024,8 +2082,8 @@ class V(IndependentSource):
     def _ss_model(self):
         return self._netmake(args='%s(t)' % self.relname.lower())
 
-    def _s_model(self, var):
-        return self._netmake(args=self.cpt.Voc.laplace()(var))
+    def _s_model(self, kind):
+        return self._netmake(args=self.cpt.Voc.laplace(), ignore_keyword=True)
 
     def _pre_initial_model(self):
 
@@ -2040,12 +2098,12 @@ class W(Dummy):
     def _stamp(self, mna):
         pass
 
-    @ property
+    @property
     def I(self):
         raise ValueError(
             'Cannot determine current through wire, use a 0 V voltage source')
 
-    @ property
+    @property
     def i(self):
         raise ValueError(
             'Cannot determine current through wire, use a 0 V voltage source')
@@ -2084,13 +2142,13 @@ def defcpt(name, base, docstring):
     classes[name] = newclass
 
 
-def make(classname, parent, namespace, defname, name, cpt_type, cpt_id,
+def make(classname, parent, namespace, name, cpt_type, cpt_id,
          string, opts_string, nodes, keyword, *args):
 
     # Create instance of component object
     newclass = classes[classname]
 
-    cpt = newclass(parent, namespace, defname, name, cpt_type, cpt_id, string,
+    cpt = newclass(parent, namespace, name, cpt_type, cpt_id, string,
                    opts_string, nodes, keyword, *args)
 
     return cpt
@@ -2100,6 +2158,7 @@ def make(classname, parent, namespace, defname, name, cpt_type, cpt_id,
 defcpt('A', Misc, 'Annotation')
 defcpt('ADC', Misc, 'ADC')
 defcpt('AM', W, 'Ammeter')
+defcpt('ANT', Misc, 'Antenna')
 
 defcpt('BAT', V, 'Battery')
 defcpt('BL', Misc, 'Block')
