@@ -1,7 +1,7 @@
 """This module provides the NetfileMixin class.  This is used for
 Netlist and Schematic to parse a netlist file.
 
-Copyright 2020--2022 Michael Hayes, UCECE
+Copyright 2020--2023 Michael Hayes, UCECE
 
 """
 
@@ -9,6 +9,7 @@ from . import grammar
 from .parser import Parser
 from .state import state
 from .componentnamer import ComponentNamer
+from warnings import warn
 from os.path import dirname, join
 
 
@@ -19,20 +20,20 @@ class NetfileMixin(object):
         self.parser = Parser(cpts, grammar, allow_anon)
         # Current namespace
         self.namespace = ''
-        self.subnetlists = {}
         self.namer = ComponentNamer()
         self.dirname = None
+        self.subnetlists = {}
 
-    def _make_anon_cpt_id(self, cpt_type):
-        """Make identifier for anonymous component"""
-
-        return self.namer.cpt_id(cpt_type + 'anon')
-
-    def _make_anon_name(self, cpt_type):
+    def _make_anon_cpt_name(self, cpt_type):
         """Make name for anonymous component"""
 
         return self.namer.name(cpt_type + 'anon', self.elements)
 
+    def _make_anon_node_name(self):
+        """Make name for anonymous node"""
+
+        return self.namer.name('nodeanon', self.nodes)
+
     def _include(self, string):
 
         parts = string.split(' ')
@@ -45,18 +46,18 @@ class NetfileMixin(object):
         if len(parts) != 4 and parts[2] != 'as':
             raise ValueError(
                 'Expecting include filename as name in %s' % string)
+
         name = parts[3]
         namespace = self.namespace
         self.namespace = name + '.' + namespace
-        self.subnetlists[self.namespace[0:-1]] = None
+        if name in self.subnetlists:
+            warn('Overriding subnetlist %s with %s for %s' %
+                 (self.subnetlists[name], parts[1], name))
+        self.subnetlists[name] = parts[1]
         ret = self._netfile_add(filename, self.namespace)
         self.namespace = namespace
         return ret
 
-    def _directive(self, string, namespace=''):
-
-        return self.parser.cpts.Directive(self, string, namespace)
-
     def _parse(self, string, namespace=''):
         """The general form is: 'Name Np Nm symbol'
         where Np is the positive node and Nm is the negative node.
