@@ -1,7 +1,7 @@
 """This module provides the NetlistMixin class.  This is used for
 Netlist and SubNetlist.
 
-Copyright 2020--2022 Michael Hayes, UCECE
+Copyright 2020--2023 Michael Hayes, UCECE
 
 """
 
@@ -27,7 +27,7 @@ from .matrix import Matrix
 from .utils import isiterable
 from .texpr import t
 from .deprecation import LcapyDeprecationWarning
-
+from .nodes import Nodes
 from . import mnacpts
 from collections import OrderedDict
 from warnings import warn
@@ -39,7 +39,7 @@ class NetlistMixin(object):
 
         self._elements = OrderedDict()
         self.namespaces = {}
-        self.nodes = AttrDict()
+        self.nodes = Nodes()
         if context is None:
             context = state.new_context()
 
@@ -100,10 +100,10 @@ class NetlistMixin(object):
         # Perhaps should prune wires, open-circuits, etc. ?
         return list(self._elements.keys())
 
-    def _dummy_node(self):
+    def _dummy_node_name(self):
         """Create a dummy node name."""
 
-        return '_' + self._make_anon_name('node')
+        return '_' + self._make_anon_node_name()
 
     def _add_ground(self, node):
 
@@ -191,15 +191,11 @@ class NetlistMixin(object):
 
         return self.cg.is_connected
 
-    def _node_add(self, node, cpt):
-
-        if node not in self.nodes:
-            self.nodes[node] = Node(self, node)
-        self.nodes[node].append(cpt)
-
     def _cpt_add(self, cpt):
 
         if cpt.name in self._elements:
+            import pdb
+            pdb.set_trace()
             warn('Overriding component %s' % cpt.name)
             # Need to search lists and update component.
             # For example, remove nodes that are only connected
@@ -213,9 +209,6 @@ class NetlistMixin(object):
 
         self._elements[cpt.name] = cpt
 
-        for node in cpt.nodenames:
-            self._node_add(node, cpt)
-
         self._namespace_add(cpt.namespace)
 
     def _namespace_add(self, namespace):
@@ -277,9 +270,12 @@ class NetlistMixin(object):
 
         if name not in self._elements:
             raise ValueError('Unknown component: ' + name)
-        self._elements.pop(name)
-        # TODO, remove nodes that are only connected
-        # to this component.
+
+        cpt = self._elements[name]
+        for node in cpt.nodes:
+            node.remove(cpt)
+
+        self._elements.pop(name, None)
         return self
 
     @property
@@ -291,7 +287,7 @@ class NetlistMixin(object):
         for elt in self.elements.values():
             if not elt.is_voltage_source:
                 continue
-            snodes.append(elt.nodenames[0:2])
+            snodes.append(elt.node_names[0:2])
 
         from .utils import merge_common
 
@@ -315,11 +311,11 @@ class NetlistMixin(object):
             if elt.nosim:
                 continue
             if elt.type == 'W':
-                enodes.add_wire(*elt.nodenames)
+                enodes.add_wire(*elt.node_names)
             elif elt.type.startswith('TL'):
-                enodes.add_wire(elt.nodenames[1], elt.nodenames[3])
+                enodes.add_wire(elt.node_names[1], elt.node_names[3])
             elif elt.type.startswith('TP'):
-                enodes.add_wire(elt.nodenames[1], elt.nodenames[3])
+                enodes.add_wire(elt.node_names[1], elt.node_names[3])
                 warn("Assuming V2' = V1' for %s" % elt.name)
             else:
                 for connections in elt.equipotential_nodes:
@@ -686,7 +682,7 @@ class NetlistMixin(object):
             if isinstance(cpt, Node):
                 Np, Nm = cpt.name, 0
             else:
-                Np, Nm = cpt.nodenames[0:2]
+                Np, Nm = cpt.node_names[0:2]
         return Np, Nm
 
     def _parse_node_args4(self, N1p, N1m, N2p, N2m, name):
@@ -1773,6 +1769,7 @@ class NetlistMixin(object):
         for net in netlist.split('\n'):
             sch.add(net)
 
+        sch.subnetlists = self.subnetlists
         self._sch = sch
         return sch
 
@@ -1867,13 +1864,13 @@ class NetlistMixin(object):
             new._add(net)
         return new
 
-    def s_model(self, var=s):
+    def s_model(self, kind='s'):
         """"Create s-domain model."""
 
         new = self._new()
 
         for cpt in self._elements.values():
-            net = cpt._s_model(var)
+            net = cpt._s_model(kind)
             new._add(net)
         return new
 
@@ -2221,8 +2218,8 @@ class NetlistMixin(object):
 
         return self._components
 
-    def describe(self):
-        """Print a message describing how circuit is solved."""
+    def description(self):
+        """Return a message describing how circuit is solved."""
 
         def describe_sources(sources):
             sources_string = ', '.join(sources)
@@ -2231,39 +2228,44 @@ class NetlistMixin(object):
             return 'sources %s' % sources_string
 
         def describe_analysis(method, sources):
-            return '%s analysis is used for %s.' % (method,
-                                                    describe_sources(sources))
+            return '%s analysis is used for %s.\n' % (method,
+                                                      describe_sources(sources))
 
         if self.is_switching:
-            print('''This has switches and thus is time variant.  Use the convert_IVP(t) method to convert to an initial value problem, specifying the time when to evaluate the switches.''')
-            return
+            return '''This has switches and thus is time variant.  Use the convert_IVP(t) method to convert to an initial value problem, specifying the time when to evaluate the switches.'''
 
         groups = self.independent_source_groups(
             transform=not self.is_time_domain)
 
         if groups == {}:
-            print('There are no non-zero independent sources so everything is zero.')
-            return
+            return 'There are no non-zero independent sources so everything is zero.\n'
 
         if self.is_IVP:
-            print('This has initial conditions for %s so is an initial value '
-                  'problem solved in the s-domain using Laplace transforms.' %
-                  ', '.join(self.ics))
+            return 'This has initial conditions for %s so is an initial value '
+            'problem solved in the s-domain using Laplace transforms.\n' \
+                % ', '.join(self.ics)
             return
 
+        s = ''
         if len(groups) > 1:
-            print('This is solved using superposition.')
+            s = 'This is solved using superposition.\n'
+
         for kind, sources in groups.items():
             if not isinstance(kind, str):
-                print(describe_analysis('Phasor', sources))
+                s += describe_analysis('Phasor', sources)
             elif kind[0] == 'n':
-                print(describe_analysis('Noise', sources))
+                s += describe_analysis('Noise', sources)
             elif kind == 'dc':
-                print(describe_analysis('DC', sources))
+                s += describe_analysis('DC', sources)
             elif kind == 's':
-                print(describe_analysis('Laplace', sources))
+                s += describe_analysis('Laplace', sources)
             elif kind in ('t', 'time'):
-                print(describe_analysis('Time-domain', sources))
+                s += describe_analysis('Time-domain', sources)
+        return s
+
+    def describe(self):
+        """Print a message describing how circuit is solved."""
+        print(self.description())
 
     def Vname(self, name):
         return Vname(name, self.kind)
