@@ -15,9 +15,7 @@ __all__ = ('NodalAnalysis', )
 
 
 class NodalAnalysis(object):
-    """
-    This is an experimental class for nodal analysis.  The API
-    is likely to change.
+    """This is for nodal analysis.
 
     >>> from lcapy import Circuit, NodalAnalysis
     >>> cct = Circuit('''
@@ -34,15 +32,15 @@ class NodalAnalysis(object):
 
     >>> na = NodalAnalysis(cct.laplace())
 
-    To display the system of equations (in matrix form) that needs to
-    be solved:
-
-    >>> na.equations().pprint()
-
     To display the equations found by applying KCL at each node:
 
     >>> na.nodal_equations().pprint()
 
+    To display the system of equations (in matrix form) that needs to
+    be solved:
+
+    >>> na.matrix_equations().pprint()
+
     """
 
     @classmethod
@@ -58,8 +56,14 @@ class NodalAnalysis(object):
         self.cct = cct
         self.cg = CircuitGraph.from_circuit(cct)
 
-        self.kind = self.cct.kind
-        if self.kind == 'super':
+        source_groups = cct.independent_source_groups()
+
+        if len(source_groups) > 1:
+            # Could convert to time domain
+            raise ValueError('Multiple independent source domains')
+        elif len(source_groups) == 1:
+            self.kind = list(source_groups)[0]
+        else:
             self.kind = 'time'
 
         self.node_prefix = node_prefix
@@ -109,8 +113,8 @@ class NodalAnalysis(object):
 
             if voltage_sources != []:
                 elt = voltage_sources[0]
-                n1 = self.cg.node_map[elt.nodenames[0]]
-                n2 = self.cg.node_map[elt.nodenames[1]]
+                n1 = self.cg.node_map[elt.node_names[0]]
+                n2 = self.cg.node_map[elt.node_names[1]]
 
                 V = elt.cpt.voltage_equation(0, self.kind)
 
@@ -119,10 +123,10 @@ class NodalAnalysis(object):
             else:
                 result = Itype(self.kind)(0)
                 for elt in self.cg.connected_cpts(node):
-                    if len(elt.nodenames) < 2:
+                    if len(elt.node_names) < 2:
                         raise ValueError('Elt %s has too few nodes' % elt)
-                    n1 = self.cg.node_map[elt.nodenames[0]]
-                    n2 = self.cg.node_map[elt.nodenames[1]]
+                    n1 = self.cg.node_map[elt.node_names[0]]
+                    n2 = self.cg.node_map[elt.node_names[1]]
                     if node == n1:
                         pass
                     elif node == n2:
