@@ -1,18 +1,24 @@
 """This module provides support for nodes.
 
-Copyright 2020--2022 Michael Hayes, UCECE
+Copyright 2020--2023 Michael Hayes, UCECE
 
 """
 
 from .immittancemixin import ImmittanceMixin
 
 
+class DummyNode:
+
+    def __init__(self, name):
+        self.name = name
+
+
 class Node(ImmittanceMixin):
 
     def __init__(self, cct, name):
 
         self.cct = cct
-        self.name = name
+        self._name = name
         self.pos = None
         self.port = False
         parts = name.split('_')
@@ -25,6 +31,38 @@ class Node(ImmittanceMixin):
     def __repr__(self):
         return "Node('%s')" % self.name
 
+    def __str__(self):
+
+        # Note, need to have float type otherwise 0 becomes empty string.
+        x = str(round(self.x, 2)).rstrip('0').rstrip('.')
+        y = str(round(self.y, 2)).rstrip('0').rstrip('.')
+
+        return '%s@(%s, %s)' % (self.name, x, y)
+
+    @property
+    def name(self):
+        return self._name
+
+    @name.setter
+    def name(self, name):
+        # Use new node name for nodes
+        self.cct.nodes[name] = self.cct.nodes.pop(self._name)
+        self._name = name
+
+    @property
+    def x(self):
+
+        if self.pos is None:
+            raise ValueError('Node position undefined')
+        return self.pos.x
+
+    @property
+    def y(self):
+
+        if self.pos is None:
+            raise ValueError('Node position undefined')
+        return self.pos.y
+
     @property
     def V(self):
         """Node voltage with respect to ground."""
@@ -58,6 +96,20 @@ class Node(ImmittanceMixin):
 
         self._connected.append(cpt)
 
+    def remove(self, cpt):
+
+        if len(self._connected) == 0:
+            raise RuntimeError('Removing node %s with 0 count' % str(self))
+
+        for c in self._connected:
+            if c.name == cpt.name:
+                self._connected.remove(c)
+                break
+        self._count -= 1
+
+        if self.count == 0:
+            self.cct.nodes._remove(self.name)
+
     @property
     def count(self):
         """Number of elements (including wires but not open-circuits and
@@ -135,3 +187,13 @@ class Node(ImmittanceMixin):
         """Return True if node is a ground"""
 
         return self.name.startswith('0')
+
+    def debug(self):
+
+        s = str(self) + ', count=%s' % self.count
+
+        names = [cpt.name for cpt in self.connected]
+
+        s += ', cpts=[%s]' % ', '.join(names) + '\n'
+
+        return s
