@@ -18,7 +18,7 @@ __all__ = ('omegaexpr', )
 
 
 class AngularFourierDomainExpression(AngularFourierDomain, Expr):
-    """Fourier domain expression or symbol (angular frequency)."""
+    """Angular Fourier domain expression or symbol."""
 
     var = omegasym
 
@@ -87,8 +87,18 @@ class AngularFourierDomainExpression(AngularFourierDomain, Expr):
         result = self.subs(Omega / dt)
         return result
 
+    def frequency_response(self, **assumptions):
+        """Convert to frequency response domain."""
+
+        return self.laplace(**assumptions).frequency_response()
+
+    def angular_frequency_response(self, **assumptions):
+        """Convert to angular frequency response domain."""
+
+        return self.laplace(**assumptions).angular_frequency_response()
+
     def laplace(self, **assumptions):
-        """Convert to Laplace domain."""
+        """Determine one-sided Laplace transform with 0- as the lower limit."""
 
         result = self.time(**assumptions).laplace()
         return result
