@@ -1,8 +1,10 @@
 """This module provides the Opts class for handling schematic options.
 
-Copyright 2021 Michael Hayes, UCECE
+Copyright 2021--2023 Michael Hayes, UCECE
 """
 
+from .latex import latex_format_label
+
 
 class Opts(dict):
 
@@ -55,7 +57,12 @@ class Opts(dict):
             elif arg in ('false', 'False'):
                 arg = False
 
-            self[key] = arg
+            if key == 'def':
+                if key not in self:
+                    self[key] = []
+                self[key].append(arg)
+            else:
+                self[key] = arg
 
     def __str__(self):
         return self.format()
@@ -116,3 +123,17 @@ class Opts(dict):
         stripped.update(self.strip_labels())
 
         return stripped
+
+    def as_list(self, ignore=None, **kwargs):
+
+        if ignore is None:
+            ignore = []
+
+        for key, val in self.items():
+            # Override with component opts
+            kwargs[key] = val
+
+        def fmt(key, val):
+            return '%s=%s' % (key, latex_format_label(val))
+
+        return [fmt(key, val) for key, val in kwargs.items() if key not in ignore and '.' not in key]
