@@ -12,7 +12,7 @@ This module performs schematic drawing using circuitikz from a netlist::
     ... W 0.2 0.2; right''')
     >>> sch.draw()
 
-Copyright 2014--2022 Michael Hayes, UCECE
+Copyright 2014--2023 Michael Hayes, UCECE
 """
 
 # Strings starting with ;; are schematic options.  They are parsed in
@@ -33,6 +33,7 @@ from .schemplacer import schemplacer
 from .opts import Opts
 from .netfile import NetfileMixin
 from .system import LatexRunner, PDFConverter, tmpfilename
+from .nodes import parse_nodes
 from os import path, remove
 from collections import OrderedDict
 from warnings import warn
@@ -60,6 +61,7 @@ def display_matplotlib(filename, dpi=PNG_DPI, scale=2):
     ax.imshow(img)
     ax.axis('off')
     fig.subplots_adjust(left=0.05, right=0.95, top=0.95, bottom=0.05)
+    fig.show()
 
 
 def png_image_size(filename):
@@ -87,7 +89,7 @@ class SchematicOpts(Opts):
              'label_ids': True,
              'annotate_values': False,
              'label_nodes': 'primary',
-             'node_label_anchor': 'south east',
+             'anchor': 'south east',
              'autoground': 'none',
              'scale': 1.0,
              'dpi': PNG_DPI,
@@ -112,6 +114,8 @@ class Schematic(NetfileMixin):
         self.dummy_node = 0
         self.context = None
         self.debug = 0
+        self.defines = {}
+        self.node_positions = {}
 
         if filename is not None:
             self.netfile_add(filename)
@@ -148,11 +152,11 @@ class Schematic(NetfileMixin):
 
         return '\n'.join([elt.__str__() for elt in self.elements.values()])
 
-    def _node_add(self, nodename, elt, auxiliary=False):
+    def _node_add(self, node_name, elt, auxiliary=False):
 
-        if nodename not in self.nodes:
-            self.nodes[nodename] = Node(nodename)
-        node = self.nodes[nodename]
+        if node_name not in self.nodes:
+            self.nodes[node_name] = Node(node_name)
+        node = self.nodes[node_name]
 
         node.append(elt)
 
@@ -167,7 +171,7 @@ class Schematic(NetfileMixin):
     def _format_name(self, cpt_type, cpt_id):
 
         name = cpt_type
-        subscript = cpt_id
+        subscript = cpt_id[1:] if cpt_id.startswith('_') else cpt_id
 
         if cpt_type == 'REL':
             name = r'\mathcal{R}'
@@ -235,10 +239,10 @@ class Schematic(NetfileMixin):
         id_label = self._format_name(cpt.type, cpt.id)
         value_label = None
 
-        if cpt.type in ('O', 'P', 'W') or id_label.find('#') != -1:
+        if cpt.type in ('A', 'O', 'P', 'W') or id_label.find('#') != -1:
             id_label = None
 
-        if cpt.type in ('S', 'SW', 'U'):
+        if cpt.type in ('A', 'S', 'SW', 'U'):
             value_label = ''
 
         unify = False
@@ -285,13 +289,14 @@ class Schematic(NetfileMixin):
             # This will prevent printing the name and value.
             unify = expr == cpt.type + cpt.id
 
-            # Currently, we only annnotate the component with the value,
-            # expression, or symbol.  If this is not specified, it
-            # defaults to the component identifier.  Note, some objects
-            # we do not want to label, such as wires and ports.
+            # Currently, we only annnotate the component with the
+            # value, expression, or symbol.  If this is not specified,
+            # it defaults to the component identifier.  Note, there
+            # are some objects we do not want to label, such as wires
+            # and ports.
         cpt.id_label = '' if id_label is None else latex_format_label(id_label)
-        cpt.value_label = cpt.id_label if value_label is None else latex_format_label(
-            value_label)
+        cpt.value_label = cpt.id_label if value_label is None \
+            else latex_format_label(value_label)
 
         if unify:
             cpt.value_label = cpt.id_label
@@ -300,18 +305,28 @@ class Schematic(NetfileMixin):
             self.hints = True
 
         if cpt.name in self.elements:
+            import pdb
+            pdb.set_trace()
             warn('Overriding component %s' % cpt.name)
             # Need to search lists and update component.
 
+        if cpt.type == 'XX' and 'nodes' in cpt.opts:
+            self.positions = parse_nodes(cpt.opts['nodes'])
+
         self.elements[cpt.name] = cpt
 
         # Perhaps don't show nodes if cpt invisible?
         if not cpt.ignore:
 
-            for node in cpt.auxiliary_node_names:
-                self._node_add(node, cpt, auxiliary=True)
+            # The auxiliary nodes are for behind the scenes
+            # use, such as finding the component centre.
+            for node_name in cpt.auxiliary_node_names:
+                pinname = node_name.split('.')[-1]
+                if pinname in cpt.required_auxiliary:
+                    self._node_add(node_name, cpt, auxiliary=True)
 
-            # Note, an auxiliary node can be trumped...
+            # The required nodes are those explicitly
+            # specified in the schematic.
             for node in cpt.required_node_names:
                 self._node_add(node, cpt, auxiliary=False)
 
@@ -323,30 +338,50 @@ class Schematic(NetfileMixin):
         elif autoground in (False, None, 'false', 'none'):
             autoground = False
 
+        for elt in self.elements.values():
+            elt.parse_nodes()
+
+        for elt in self.elements.values():
+            elt.check_nodes()
+
         if autoground:
             for elt in self.elements.values():
                 elt.autoground(autoground)
 
-        for elt in self.elements.values():
-            elt.split_nodes()
-
         for elt in self.elements.values():
             elt.setup()
 
-        for nodename, node in self.nodes.items():
-            if node.ref is not None:
-                continue
-            if node.cptname is not None and not node.implicit:
-                raise ValueError('Unreferenced pin connection %s for %s' % (
-                    node.name, node.elt_list))
+        # Cannot split nodes until know all the nodes.
+        for elt in self.elements.values():
+            elt.process_implicit_nodes()
 
-    def make_graphs(self, debug=0):
+    def make_graphs(self, debug=0, **kwargs):
 
+        # This is only used by schtex for debugging
+        autoground = kwargs.get('autoground', False)
+        self._setup(autoground)
         placer = schemplacer(self.elements, self.nodes, 'graph', debug)
         placer._make_graphs()
         return placer.xgraph, placer.ygraph
 
-    def _positions_calculate(self, method='graph', debug=False):
+    def _positions_calculate(self, method='graph', debug=False, **kwargs):
+
+        autoground = kwargs.get('autoground', False)
+        self._setup(autoground)
+
+        if self.node_positions != {}:
+            for k, v in self.node_positions.items():
+                # Opamp ground node not included
+                if k in self.nodes:
+                    self.nodes[k].pos = v
+
+            nodes = [node for node in self.nodes.values() if not node.is_split]
+
+            x = [node.pos.x for node in nodes if not node.auxiliary]
+            y = [node.pos.y for node in nodes if not node.auxiliary]
+            self.width = max(x) - min(x)
+            self.height = max(y) - min(y)
+            return False
 
         if self.debug & 4:
             print('Creating graphs')
@@ -355,22 +390,90 @@ class Schematic(NetfileMixin):
             print('Solving graphs')
         self.width, self.height = placer.solve(self.node_spacing)
 
+        return True
+
+    def _write_coordinates(self):
+
+        s = ''
+        for n in self.nodes.values():
+            s += r'  \coordinate (%s) at (%s);''\n' % (n.s, n.pos)
+        return s
+
+    def _draw_components(self, **kwargs):
+
+        s = ''
+        for elt in self.elements.values():
+            if elt.ignore:
+                continue
+            if elt.directive:
+                for key, val in elt.opts.items():
+                    # Local opts overrides global opts in kwargs
+                    kwargs[key] = val
+
+            s += elt.draw(**kwargs)
+        return s
+
+    def _draw_nodes(self, **kwargs):
+
+        s = ''
+        for elt in self.elements.values():
+            if elt.ignore:
+                continue
+            if elt.directive:
+                for key, val in elt.opts.items():
+                    # Local opts overrides global opts in kwargs
+                    kwargs[key] = val
+
+            s += elt.draw_nodes(**kwargs)
+            s += elt.draw_pins()
+        return s
+
+    def _draw_node_labels(self, **kwargs):
+
+        s = ''
+        for elt in self.elements.values():
+            if elt.ignore:
+                continue
+            if elt.directive:
+                for key, val in elt.opts.items():
+                    # Local opts overrides global opts in kwargs
+                    kwargs[key] = val
+
+            s += elt.draw_node_labels(**kwargs)
+        return s
+
+    def _draw_help_lines(self, help_lines, help_lines_color):
+
+        start = Pos(-0.5, -0.5) * self.node_spacing
+        stop = Pos(self.width + 0.5, self.height + 0.5) * self.node_spacing
+
+        s = r'  \draw[help lines, %s] (%s) grid [xstep=%s, ystep=%s] (%s);''\n' % (
+            help_lines_color, start, help_lines, help_lines, stop)
+
+        return s
+
     def _tikz_draw(self, style_args='', **kwargs):
 
+        if (self.debug & 64):
+            import pdb
+            pdb.set_trace()
+
         method = kwargs.pop('method', 'graph')
-        autoground = kwargs.get('autoground', False)
+        preamble = kwargs.pop('preamble', None)
+        postamble = kwargs.pop('postamble', None)
+        options = kwargs.pop('options', None)
+        help_lines = float(kwargs.pop('help_lines', 0))
+        help_lines_color = kwargs.pop('help_lines_color', 'blue')
 
-        self._setup(autoground)
-
-        self._positions_calculate(method, self.debug)
+        self._positions_calculate(method, self.debug, **kwargs)
 
         # Note, scale does not scale the font size.
         opts = ['scale=%.2f' % self.scale,
                 'transform shape',
                 '/tikz/circuitikz/bipoles/length=%.2fcm' % self.cpt_size]
         opts.append(style_args)
-        if 'options' in kwargs:
-            opts.append(kwargs.pop('options'))
+        if options is not None:
+            opts.append(options)
 
         global_options = ['font', 'voltage_dir', 'color']
         for opt in global_options:
@@ -380,51 +483,28 @@ class Schematic(NetfileMixin):
         s = r'\begin{tikzpicture}[%s]''\n' % ', '.join(opts)
 
         # Add preamble
-        if 'preamble' in kwargs:
-            s += '  ' + kwargs.pop('preamble') + '\n'
+        if preamble:
+            s += '  ' + preamble + '\n'
 
-        help_lines = float(kwargs.pop('help_lines', 0))
-        help_lines_color = kwargs.pop('help_lines_color', 'blue')
         if help_lines != 0:
-            start = Pos(-0.5, -0.5) * self.node_spacing
-            stop = Pos(self.width + 0.5, self.height + 0.5) * self.node_spacing
+            s += self._draw_help_lines(help_lines, help_lines_color)
 
-            s += r'  \draw[help lines, %s] (%s) grid [xstep=%s, ystep=%s] (%s);''\n' % (
-                help_lines_color, start, help_lines, help_lines, stop)
+        # Write coordinates.
+        s += self._write_coordinates()
 
-        # Write coordinates.  TODO, not all coordinates are needed
-        # so those can be weeded out to simplify the generated file.
-        for n in self.nodes.values():
-            s += r'  \coordinate (%s) at (%s);''\n' % (n.s, n.pos)
+        # Draw components
+        s += self._draw_components(**kwargs)
 
-        # Keyword args for second pass
-        kwargs2 = kwargs.copy()
+        # Draw nodes (do not do concurrently with _draw_components
+        # since get a short piece of wire through ocirc nodes)
+        s += self._draw_nodes(**kwargs)
 
-        # Pass 1: Draw components
-        for elt in self.elements.values():
-            if elt.ignore:
-                continue
-            if elt.directive:
-                for key, val in elt.opts.items():
-                    # Directive overrides kwargs
-                    kwargs[key] = val
-
-            s += elt.draw(**kwargs)
-            s += elt.draw_nodes(**kwargs)
-            s += elt.draw_pins()
-
-        # Pass 2: Add the node labels
-        for elt in self.elements.values():
-            if elt.ignore:
-                continue
-            if elt.directive:
-                for key, val in elt.opts.items():
-                    kwargs2[key] = val
-            s += elt.draw_node_labels(**kwargs2)
+        # Draw node labels
+        s += self._draw_node_labels(**kwargs)
 
         # Add postamble
-        if 'postamble' in kwargs:
-            s += '  ' + kwargs.pop('postamble') + '\n'
+        if postamble:
+            s += '  ' + postamble + '\n'
 
         s += r'\end{tikzpicture}''\n'
 
@@ -563,8 +643,7 @@ class Schematic(NetfileMixin):
              'alpha' to label nodes starting with a letter,
              'pins' to label nodes that are pins on a chip,
              'all' to label all nodes,
-             'none' to label no nodes
-           'node_label_anchor': where to position node label (default south east)
+           'anchor': where to position node label (default south east)
            'include': name of file to include before \\begin{document}
            'style': 'american', 'british', or 'european'
            'scale': schematic scale factor, default 1.0
@@ -575,14 +654,15 @@ class Schematic(NetfileMixin):
            'debug': non-zero to display debug information
         """
 
-        # None means don't care, so remove
-        for key in list(kwargs.keys()):
+        # None means don't care, so remove.  Use list so can remove
+        # entries from kwargs
+        for key in list(kwargs):
             if kwargs[key] is None:
                 kwargs.pop(key)
 
-        # Remove options that may be overridden
+        # Remove options that may be overridden by arguments to draw
         for elt in self.elements.values():
-            for key in list(elt.opts.keys()):
+            for key in list(elt.opts):
                 if key in kwargs:
                     elt.opts.remove(key)
 
@@ -592,15 +672,19 @@ class Schematic(NetfileMixin):
             if key not in kwargs:
                 kwargs[key] = val
 
-        # Global options (usually at end of netlist for historical reasons)
-        # These can be overwritten
-        for eltname in self.elements:
-            elt = self.elements[eltname]
+        # Global options (at end of netlist for historical reasons)
+        # These overwrite the default options.  Skip blank lines.
+        for elt_name in reversed(self.elements):
+            elt = self.elements[elt_name]
             if not elt.directive:
+                break
+            if elt.string.strip().startswith(';;'):
                 continue
-            for key, val in elt.opts.items():
-                # val is a str
-                kwargs[key] = val
+            if elt.string.strip().startswith(';'):
+                for key, val in elt.opts.items():
+                    # val is a str
+                    kwargs[key] = val
+                break
 
         def in_ipynb():
             try:
