@@ -24,10 +24,10 @@ class LcapyTester(unittest.TestCase):
   \coordinate (2) at (2,0);
   \coordinate (3) at (2.5,0);
   \coordinate (0) at (4.5,0);
-  \draw[] (1) to [R,l_=$1\,\mbox{$\Omega$}$,,,,n=R1] (2);
+  \draw (1) to [R, l_=$1\,\mbox{$\Omega$}$, n=R1] (2);
+  \draw[-] (2) to (3);
+  \draw (3) to [C, l_=$2\,\mbox{F}$, n=C1] (0);
   \draw (1) node[ocirc] {};
-  \draw[-, , ] (2) to (3);
-  \draw[] (3) to [C,l_=$2\,\mbox{F}$,,,,n=C1] (0);
   \draw (0) node[ocirc] {};
 \end{tikzpicture}
 \end{document}"""
@@ -54,20 +54,14 @@ C 1 3 3; right""")
   \coordinate (1) at (0,0);
   \coordinate (2) at (2,0);
   \coordinate (3) at (4,0);
-  \draw[] (1) to [R,l_={$R_{1}$}{=$1\,\mbox{$\Omega$}$},,,,n=R1] (2);
+  \draw (1) to [R, l_={$R_{1}$}{=$1\,\mbox{$\Omega$}$}, n=R1] (2);
+  \draw (2) to [R, l_={$R_{2}$}{=$2\,\mbox{$\Omega$}$}, n=R2] (3);
+  \draw (1) to [C, l_={$C$}{=$3\,\mbox{F}$}, n=C] (3);
   \draw (1) node[circ] {};
   \draw (2) node[circ] {};
-  \draw[] (2) to [R,l_={$R_{2}$}{=$2\,\mbox{$\Omega$}$},,,,n=R2] (3);
-  \draw (2) node[circ] {};
-  \draw (3) node[circ] {};
-  \draw[] (1) to [C,l_={$C$}{=$3\,\mbox{F}$},,,,n=C] (3);
-  \draw (1) node[circ] {};
   \draw (3) node[circ] {};
   \draw[anchor=south east] (1) node {1};
   \draw[anchor=south east] (2) node {2};
-  \draw[anchor=south east] (2) node {2};
-  \draw[anchor=south east] (3) node {3};
-  \draw[anchor=south east] (1) node {1};
   \draw[anchor=south east] (3) node {3};
 \end{tikzpicture}
 \end{document}"""
