@@ -15,7 +15,7 @@ from .expr import symbol, expr, ExprDict, ExprList
 from .differenceequation import DifferenceEquation
 from .seqexpr import SequenceExpression
 from .zseq import ZDomainSequence
-from .functions import sqrt, exp
+from .functions import sqrt, exp, Function
 from sympy import Eq, div, limit, oo, Sum
 
 
@@ -261,13 +261,20 @@ class ZDomainExpression(ZDomain, SequenceExpression):
                                                                **kwargs)
 
     def inverse_bilinear_transform(self):
+        """Approximate z = exp(s * dt)
+
+        by z = (1 + 0.5 * dt * s) / (1 - 0.5 * dt * s)"""
+
+        # TODO: add frequency warping as an option
 
         from .symbols import s
         from .discretetime import dt
 
-        # z = exp(s * dt) gives the exact solution
+        a = s * dt / 2
 
-        return self.subs((1 + s * dt / 2) / (1 - s * dt / 2))
+        scale = 1 if self.is_ratio else dt
+
+        return self.subs((1 + a) / (1 - a)) * scale
 
     def DFT(self, N=None, evaluate=True, **assumptions):
         """Determine DFT.
@@ -360,9 +367,11 @@ class ZDomainExpression(ZDomain, SequenceExpression):
         `form` can be 'fir' or 'iir' ('direct form I').
         """
 
+        from .symbols import n
+
         H = self
-        x = nexpr('%s(n)' % inputsym)
-        y = nexpr('%s(n)' % outputsym)
+        x = Function(inputsym)(n)
+        y = Function(outputsym)(n)
 
         X = x.ZT()
         Y = y.ZT()
@@ -382,8 +391,13 @@ class ZDomainExpression(ZDomain, SequenceExpression):
         return DifferenceEquation(lhs, rhs, inputsym, outputsym)
 
     def dlti_filter(self, normalize_a0=True):
-        """Create discrete-time linear time-invariant filter from discrete-time
-        transfer function."""
+        """Create discrete-time linear time-invariant filter from
+        discrete-time transfer function.
+
+        If `normalize_a0` is `True` (default) all the coefficients
+        are normalized so a0 = 1.
+
+        """
 
         # TODO, perhaps add only to DiscreteTimeDomainTransfer?
 
@@ -403,26 +417,19 @@ class ZDomainExpression(ZDomain, SequenceExpression):
         bn = ExprList((len(dn) - len(nn)) * [0] + nn)
         an = dn
 
+        # Remove trailing zero coefficients.  Could call cancel before
+        # determing coeffs to reduce order of numerator and denominator.
+        while an[-1] == 0:
+            an = an[0:-1]
+        while bn[-1] == 0:
+            bn = bn[0:-1]
+
         if normalize_a0:
             bn = [bx / an[0] for bx in bn]
             an = [ax / an[0] for ax in an]
 
-        lpf = DLTIFilter(bn, an)
-        return lpf
-
-    def inverse_bilinear_transform(self):
-        """Approximate z = exp(s * dt)
-
-        by z = (1 + 0.5 * dt * s) / (1 - 0.5 * dt * s)"""
-
-        # TODO: add frequency warping as an option
-
-        from .symbols import s
-        from .discretetime import dt
-
-        a = s * dt / 2
-
-        return self.subs((1 + a) / (1 - a))
+        fil = DLTIFilter(bn, an)
+        return fil
 
     def zdomain(self, **assumptions):
         return self
